/*---------------------------------------------------------+
I                                                          I
I  Module            : rs232                               I
I                                                          I
I  Creation          : 22 Mars      1991                   I
I  Derniere revision : 14 Septembre 1992                   I
I                                                          I
I  Auteur            : G. Grimonprez                       I
I                                                          I
I                                                          I
I  INTERFACE                                               I
I                                                          I
I Tous les #define sauf MAX_TAMPON et COEF_TAUX            I
I                                                          I
I void init_controleur(                                    I
I      int mode_reception,     INTERRUPT ou POLLING        I
I      int numero_port,        COM1 ou COM2                I
I      unsigned long int baud, 1 a 115200                  I
I      int nb_bits_donnee,     5 a 8                       I
I      int nb_bits_arret,      1 ou 2                      I
I      int parite,             SANS, PAIRE ou IMPAIRE      I
I      int max_time            Nombre de secondes          I
I		     );                                    I
I                                                          I
I BOOLEAN  recevoir_octet(char *c);                        I
I                                                          I
I BOOLEAN  recevoir_octet_timeout(char *c);                I
I                                                          I
I char     reception_octet(void);                          I
I                                                          I
I BOOLEAN  emettre_octet(char c);                          I
I                                                          I
I BOOLEAN  emettre_octet_timeout(char c);                  I
I                                                          I
I void     emission_octet(char c);                         I
I                                                          I
I void     restaure_controleur(void);                      I
I                                                          I
I int      taux_remplissage(void)                          I
I          (seulement si mode_reception==INTERRUPT)        I
+---------------------------------------------------------*/

/*---------------------------------------------------------+
I                            INCLUDES                      I
+---------------------------------------------------------*/

#include <stdio.h>
#include <dos.h>
#include <time.h>
#include <process.h>
/*--------------------------------------------------------I
I           Define standard                               I
I                                                         I
I Creation          :  22 Mars 1991                       I
I Derniere revision :  14 Septembre 1992                  I
I                                                         I
I Auteur            : G. Grimonprez                       I
I                                                         I
---------------------------------------------------------*/

#ifndef BOOLEAN
#define BOOLEAN     char
#endif

#ifndef TRUE
#define TRUE        1
#endif

#ifndef FALSE
#define FALSE       0
#endif

/*---------------------------------------------------------+
I                            DEFINES                       I
+---------------------------------------------------------*/

#define SANS        0
#define PAIRE       1
#define IMPAIRE     2
#define COM1        1
#define COM2        2
#define INTERRUPT   1
#define POLLING     0

#define NUL         0
#define SOH         1
#define STX         2
#define ETX         3
#define EOT         4
#define ENQ         5
#define ACK         6
#define BEL         7
#define BS          8
#define HT          9
#define LF          10
#define VT          11
#define FF          12
#define CR          13
#define SO          14
#define SI          15
#define DLE         16
#define DC1         17
#define DC2         18
#define DC3         19
#define DC4         20
#define NAK         21
#define SYN         22
#define ETB         23
#define CAN         24
#define EM          25
#define SUB         26
#define ESC         27
#define FS          28
#define GS          29
#define RS          30
#define US          31
#define DEL         127

#define MAX_TAMPON  8192
#define COEF_TAUX   100L/(long int)MAX_TAMPON

/*---------------------------------------------------------+
I                      PROTOTYPES                          I
+---------------------------------------------------------*/

int  taux_remplissage(void);
void interrupt spg_int_port_serie(void);
void init_interruption(void);
void init_vitesse(unsigned long int bauds);
void init_controle_ligne(int nb_bits_donnee,int nb_bits_arret,int parite);
void init_controle_modem(void);
void init_interruption_sur_le_port(void);
void err_init(char *message);
void init_controleur(int mode_reception,int numero_port,
		     unsigned long int baud,int nb_bits_donnee,
		     int nb_bits_arret,int parite,int max_time);
BOOLEAN  recevoir_octet(char *c);
BOOLEAN  recevoir_octet_timeout(char *c);
char     reception_octet(void);
BOOLEAN  emettre_octet(char c);
BOOLEAN  emettre_octet_timeout(char c);
void     emission_octet(char c);
void restaure_controleur(void);

/*---------------------------------------------------------+
I                      DECLARATIONS GLOBALES               I
+---------------------------------------------------------*/

int
     b_interrupt,
     registre_donnees,
     registre_controle_interruption,
     registre_etat_interruption,
     registre_controle_ligne,
     registre_controle_modem,
     registre_etat_ligne,
     registre_etat_modem,
     numero_interruption_port_serie,
     masque_8259,
     adr_port,
     max_time_out;

char tampon[MAX_TAMPON];
int  etampon,
     ltampon,
     ntampon;

void interrupt (*old_pointer_interrupt)();

/*---------------------------------------------------------+
I                           FONCTIONS                      I
+---------------------------------------------------------*/

/*----- ERREUR_FATALE ----------------------*/

void erreur_fatale(char msg[])
{
 printf("\nERREUR FATALE %s\n",msg);
 exit(1);
}

/*----- TAUX_REMPLISSAGE ---------------------------------*/

int taux_remplissage(void)
{

 if(!b_interrupt)
    err_init("Appel de taux_remplissage : mode_reception != INTERRUPT");
 return (int)(ntampon*COEF_TAUX);
}

/*----- SPG_INT_PORT_SERIE -------------------------------*/

void  interrupt spg_int_port_serie(void)
{
 tampon[etampon]=inportb(registre_donnees);
 etampon++;
 if(etampon>=MAX_TAMPON) etampon=0;
 ntampon++;
 outportb(0x20,0x20); /* Signal au controleur 8259 que l'interruption est
			traitee */
}

/*----- INIT_INTERRUPTION --------------------------------*/

void init_interruption(void)
{

 old_pointer_interrupt=getvect(numero_interruption_port_serie);
 setvect(numero_interruption_port_serie,spg_int_port_serie);
 outportb(0x21,inportb(0x21) & masque_8259); /* on signale au 8259 qu'il va
			     recevoir des interruptions venant du port */
 etampon=0;
 ltampon=0;
 ntampon=0;
}

/*----- RESTAURE_CONTROLEUR ------------------------------*/

void restaure_controleur(void)
{

 if(b_interrupt)
    setvect(numero_interruption_port_serie,old_pointer_interrupt);
}

/*----- INIT_VITESSE -------------------------------------*/

void init_vitesse(unsigned long int b)
{
 int diviseur_debit;

 outportb(registre_controle_ligne,0x80);
 diviseur_debit=(int)(115200L / b);
 outportb(registre_donnees,diviseur_debit % 256);
 outportb(registre_controle_interruption,diviseur_debit / 256);
 outportb(registre_controle_ligne,0);
}

/*----- INIT_CONTROLE_LIGNE ------------------------------*/

void init_controle_ligne(int nb_bits_donnee,int nb_bits_arret,int parite)
{
 char valeur;

 valeur=nb_bits_donnee-5;
 if(nb_bits_arret!=1) valeur+=4;
 if(parite!=SANS) {
    valeur+=8;
    if(parite==PAIRE) valeur+=16;
    }
 outportb(registre_controle_ligne,valeur);
}

/*----- INIT_CONTROLE_MODEM ------------------------------*/

void init_controle_modem(void)
{

 if(b_interrupt) outportb(registre_controle_modem,11);
 else            outportb(registre_controle_modem,3);
}

/*----- INIT_INTERRUPTION_SUR_LE_PORT --------------------*/

void init_interruption_sur_le_port(void)
{

 if(b_interrupt) {
    outportb(registre_controle_interruption,1);
    init_interruption();
    }
 else
    outportb(registre_controle_interruption,0);
}

/*----- ERR_INIT ----------------------------------------*/

void err_init(char *message)
{

 printf("\nERREUR INIT_CONTROLEUR: %s\n",message);
 exit(1);
}

/*----- INIT_CONTROLEUR ---------------------------------*/

void init_controleur(int mode_reception,int numero_port,
		     unsigned long int baud,int nb_bits_donnee,
		     int nb_bits_arret,int parite,int max_time)
{

 printf("\nINIT CONTROLEUR:\n\n");
 switch(mode_reception) {
        case POLLING:
             printf("     reception des caracteres par POLLING\n");
             break;
        case INTERRUPT:
             printf("     reception des caracteres par INTERRUPTION\n");
             printf("               %d octets alloues pour le tampon\n",
	            MAX_TAMPON);
             break;
        default:
             err_init("mode reception different de POLLING et INTERRUPT");
             break;
        }
 switch(numero_port) {
        case COM1:
             printf("     port COM1\n");
             break;
        case COM2:
             printf("     port COM2\n");
             break;
        default:
             err_init("numero de port different de COM1 et COM2");
             break;
        }
 printf("     Bauds: %lu\n",baud);
 switch(nb_bits_donnee) {
        case 5:
        case 6:
        case 7:
        case 8:
               printf("     Nombre de bits de donnees: %d\n",nb_bits_donnee);
               break;
        default:
               err_init("nb bits de donnees < 5 ou > 8");
               break;
        }
 switch(nb_bits_arret) {
        case 1:
        case 2:
               printf("     Nombre de bits d'arret: %d\n",nb_bits_arret);
               break;
        default:
               err_init("nb bits d'arret different de 1 et 2");
               break;
        }
switch(parite) {
       case SANS:
               printf("     sans parite\n");
               break;
       case PAIRE:
               printf("     parite paire\n");
               break;
       case IMPAIRE:
               printf("     parite impaire\n");
               break;
       default:
               err_init("parite differente de SANS et PAIRE et IMPAIRE");
               break;
        }
 printf("     Valeur du time_out %d\n\n",max_time);
 b_interrupt=mode_reception;
 max_time_out=max_time;
 if(numero_port==1) {
    adr_port=0x3f8;
    numero_interruption_port_serie=0x0c;
    masque_8259=0xef;
    }
 else {
    adr_port=0x2f8;
    numero_interruption_port_serie=0x0b;
    masque_8259=0xf7;
    }
 registre_donnees              =adr_port;
 registre_controle_interruption=adr_port+1;
 registre_etat_interruption    =adr_port+2;
 registre_controle_ligne       =adr_port+3;
 registre_controle_modem       =adr_port+4;
 registre_etat_ligne           =adr_port+5;
 registre_etat_modem           =adr_port+6;
 inportb(registre_donnees); /* pour debloquer eventuellement le port */
 init_vitesse(baud);
 init_controle_ligne(nb_bits_donnee,nb_bits_arret,parite);
 init_controle_modem();
 init_interruption_sur_le_port();
}

/*----- RECEVOIR_OCTET -----------------------------------*/

BOOLEAN recevoir_octet(char *c)
{

 if(b_interrupt) {
   if(etampon==ltampon) return FALSE;
   *c=tampon[ltampon];
   ltampon++;
   if(ltampon>=MAX_TAMPON) ltampon=0;
   ntampon--;
   return TRUE;
   }
 if((inportb(registre_etat_ligne) & 1)==1) {
    *c=inportb(registre_donnees);
    return TRUE;
    }
 return FALSE;
}

/*----- EMETTRE_OCTET ------------------------------------*/

BOOLEAN emettre_octet(char c)
{

 if((inportb(registre_etat_ligne) & 32)==32) {
     outportb(registre_donnees,c);
     return TRUE;
     }
 return FALSE;
}

/*----- EMETTRE_OCTET_TIMEOUT ----------------------------*/

BOOLEAN emettre_octet_timeout(char c)
{
 time_t top,w;

 top=time(&top);
 do {
    if(time(&w)-top>max_time_out) return FALSE;
    }
 while(!emettre_octet(c));
 return TRUE;
}

/*----- RECEVOIR_OCTET_TIMEOUT ---------------------------*/

BOOLEAN recevoir_octet_timeout(char *c)
{
 time_t top,w;

 top=time(&top);
 do {
    if(time(&w)-top>max_time_out) return FALSE;
    } while(!recevoir_octet(c));
 return TRUE;
}

/*----- RECEPTION_OCTET ----------------------------------*/

char reception_octet(void)
{
 char c;

 if(!recevoir_octet_timeout(&c)) erreur_fatale("Timeout en reception");
 return c;
}

/*----- EMISSION_OCTET -----------------------------------*/

void emission_octet(char c)
{

 if(!emettre_octet_timeout(c)) erreur_fatale("Time Out en emission");
}
void dump(char *message,unsigned char *pointeur,int nb_octets);

/*----- DUMP ------------------------------------------------------*/

void dump(char *message,unsigned char *pointeur,int nb_octets)
{
 int i;
 char trad[17];
 unsigned char c;

 i=0;
 trad[16]=0;
 printf("%s\n",message);
 printf("%08x ",pointeur);
 while(nb_octets) {
     c=*pointeur;
     pointeur++;
     printf("%02x ",c);
     if(c>=' ' && c<=126) trad[i]=c;
     else trad[i]='.';
     i++;
     nb_octets--;
     if(i==16) {
	printf(" %s",trad);
	if(nb_octets) printf("\n%08x ",pointeur);
	i=0;
	}
     }
 if(i) {
    trad[i]=0;
    printf(" %s",trad);
    }
 printf("\n");
}
